@@ -8,8 +8,9 @@ const Main = imports.ui.main;
 const Params = imports.misc.params;
 const ViewSelector = imports.ui.viewSelector;
 const WorkspaceThumbnail = imports.ui.workspaceThumbnail;
+const Overview = imports.ui.overview;
 
-var SIDE_CONTROLS_ANIMATION_TIME = 160;
+var SIDE_CONTROLS_ANIMATION_TIME = Overview.ANIMATION_TIME;
 
 function getRtlSlideDirection(direction, actor) {
     let rtl = actor.text_direction == Clutter.TextDirection.RTL;
@@ -422,6 +423,7 @@ class ControlsManager extends St.Widget {
         let activeWorkspaceIndex = workspaceManager.get_active_workspace_index();
 
         this._workspaceAdjustment = new St.Adjustment({
+            actor: this,
             value: activeWorkspaceIndex,
             lower: 0,
             page_increment: 1,
@@ -453,8 +455,6 @@ class ControlsManager extends St.Widget {
         this._group.add_child(this.viewSelector);
         this._group.add_actor(this._thumbnailsSlider);
 
-        layout.connect('allocation-changed', this._updateWorkspacesGeometry.bind(this));
-
         Main.overview.connect('showing', this._updateSpacerVisibility.bind(this));
 
         this.connect('destroy', this._onDestroy.bind(this));
@@ -477,26 +477,6 @@ class ControlsManager extends St.Widget {
         this._workspaceAdjustment.value = activeIndex;
     }
 
-    _updateWorkspacesGeometry() {
-        let [x, y] = this.get_transformed_position();
-        let [width, height] = this.get_transformed_size();
-        let geometry = { x, y, width, height };
-
-        let spacing = this.get_theme_node().get_length('spacing');
-        let dashWidth = this._dashSlider.getVisibleWidth() + spacing;
-        let thumbnailsWidth = this._thumbnailsSlider.getNonExpandedWidth() + spacing;
-
-        geometry.width -= dashWidth;
-        geometry.width -= thumbnailsWidth;
-
-        if (this.get_text_direction() == Clutter.TextDirection.LTR)
-            geometry.x += dashWidth;
-        else
-            geometry.x += thumbnailsWidth;
-
-        this.viewSelector.setWorkspacesFullGeometry(geometry);
-    }
-
     _setVisibility() {
         // Ignore the case when we're leaving the overview, since
         // actors will be made visible again when entering the overview
