@@ -346,6 +346,8 @@ struct _App
 
   GSList *notify_appointments; /* CalendarAppointment *, for EventsAdded */
   GSList *notify_ids; /* gchar *, for EventsRemoved */
+  guint events_added_timeout_id;
+  guint events_removed_timeout_id;
 
   GSList *live_views;
 };
@@ -368,19 +370,24 @@ app_update_timezone (App *app)
     }
 }
 
-static void
-app_notify_events_added (App *app)
+static gboolean
+on_app_schedule_events_added_cb (gpointer user_data)
 {
+  App *app = user_data;
   GVariantBuilder builder, extras_builder;
   GSList *events, *link;
 
+  if (g_source_is_destroyed (g_main_current_source ()))
+    return FALSE;
+
   events = g_slist_reverse (app->notify_appointments);
   app->notify_appointments = NULL;
+  app->events_added_timeout_id = 0;
 
   print_debug ("Emitting EventsAddedOrUpdated with %d events", g_slist_length (events));
 
   if (!events)
-    return;
+    return FALSE;
 
   /* The a{sv} is used as an escape hatch in case we want to provide more
    * information in the future without breaking ABI
@@ -421,21 +428,41 @@ app_notify_events_added (App *app)
   g_variant_builder_clear (&builder);
 
   g_slist_free_full (events, calendar_appointment_free);
+
+  return FALSE;
 }
 
 static void
-app_notify_events_removed (App *app)
+app_schedule_events_added (App *app)
 {
+  print_debug ("Scheduling EventsAddedOrUpdated");
+  if (app->events_added_timeout_id == 0)
+    {
+      app->events_added_timeout_id = g_timeout_add_seconds (2,
+                                                            on_app_schedule_events_added_cb,
+                                                            app);
+      g_source_set_name_by_id (app->events_added_timeout_id, "[gnome-shell] on_app_schedule_events_added_cb");
+    }
+}
+
+static gboolean
+on_app_schedule_events_removed_cb (gpointer user_data)
+{
+  App *app = user_data;
   GVariantBuilder builder;
   GSList *ids, *link;
 
+  if (g_source_is_destroyed (g_main_current_source ()))
+    return FALSE;
+
   ids = app->notify_ids;
   app->notify_ids = NULL;
+  app->events_removed_timeout_id = 0;
 
   print_debug ("Emitting EventsRemoved with %d ids", g_slist_length (ids));
 
   if (!ids)
-    return;
+    return FALSE;
 
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
   for (link = ids; link; link = g_slist_next (link))
@@ -456,7 +483,20 @@ app_notify_events_removed (App *app)
 
   g_slist_free_full (ids, g_free);
 
-  return;
+  return FALSE;
+}
+
+static void
+app_schedule_events_removed (App *app)
+{
+  print_debug ("Scheduling EventsRemoved");
+  if (app->events_removed_timeout_id == 0)
+    {
+      app->events_removed_timeout_id = g_timeout_add_seconds (2,
+                                                              on_app_schedule_events_removed_cb,
+                                                              app);
+      g_source_set_name_by_id (app->events_removed_timeout_id, "[gnome-shell] on_app_schedule_events_removed_cb");
+    }
 }
 
 static void
@@ -506,7 +546,7 @@ app_process_added_modified_objects (App *app,
   g_clear_object (&cal_client);
 
   if (app->notify_appointments)
-    app_notify_events_added (app);
+    app_schedule_events_added (app);
 }
 
 static void
@@ -570,7 +610,7 @@ on_objects_removed (ECalClientView *view,
   g_clear_object (&client);
 
   if (app->notify_ids)
-    app_notify_events_removed (app);
+    app_schedule_events_removed (app);
 }
 
 static gboolean
@@ -655,10 +695,35 @@ app_stop_view (App *app,
       g_signal_handlers_disconnect_by_func (view, on_objects_removed, app);
 }
 
+static void
+app_notify_has_calendars (App *app)
+{
+  GVariantBuilder dict_builder;
+
+  g_variant_builder_init (&dict_builder, G_VARIANT_TYPE ("a{sv}"));
+  g_variant_builder_add (&dict_builder, "{sv}", "HasCalendars",
+                         g_variant_new_boolean (app_has_calendars (app)));
+
+  g_dbus_connection_emit_signal (app->connection,
+                                 NULL,
+                                 "/org/gnome/Shell/CalendarServer",
+                                 "org.freedesktop.DBus.Properties",
+                                 "PropertiesChanged",
+                                 g_variant_new ("(sa{sv}as)",
+                                                "org.gnome.Shell.CalendarServer",
+                                                &dict_builder,
+                                                NULL),
+                                 NULL);
+  g_variant_builder_clear (&dict_builder);
+}
+
 static void
 app_update_views (App *app)
 {
   GSList *link, *clients;
+  gboolean had_views, has_views;
+
+  had_views = app->live_views != NULL;
 
   for (link = app->live_views; link; link = g_slist_next (link))
     {
@@ -683,31 +748,14 @@ app_update_views (App *app)
         app->live_views = g_slist_prepend (app->live_views, view);
     }
 
+  has_views = app->live_views != NULL;
+
+  if (has_views != had_views)
+    app_notify_has_calendars (app);
+
   g_slist_free_full (clients, g_object_unref);
 }
 
-static void
-app_notify_has_calendars (App *app)
-{
-  GVariantBuilder dict_builder;
-
-  g_variant_builder_init (&dict_builder, G_VARIANT_TYPE ("a{sv}"));
-  g_variant_builder_add (&dict_builder, "{sv}", "HasCalendars",
-                         g_variant_new_boolean (app_has_calendars (app)));
-
-  g_dbus_connection_emit_signal (g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL),
-                                 NULL,
-                                 "/org/gnome/Shell/CalendarServer",
-                                 "org.freedesktop.DBus.Properties",
-                                 "PropertiesChanged",
-                                 g_variant_new ("(sa{sv}as)",
-                                                "org.gnome.Shell.CalendarServer",
-                                                &dict_builder,
-                                                NULL),
-                                 NULL);
-  g_variant_builder_clear (&dict_builder);
-}
-
 static void
 on_client_appeared_cb (CalendarSources *sources,
                        ECalClient *client,
@@ -826,6 +874,9 @@ app_free (App *app)
 {
   GSList *ll;
 
+  g_clear_handle_id (&app->events_added_timeout_id, g_source_remove);
+  g_clear_handle_id (&app->events_removed_timeout_id, g_source_remove);
+
   for (ll = app->live_views; ll != NULL; ll = g_slist_next (ll))
     {
       ECalClientView *view = E_CAL_CLIENT_VIEW (ll->data);
