@@ -78,6 +78,7 @@ struct _ShellApp
                           * want (e.g. it will be of TYPE_NORMAL from
                           * the way shell-window-tracker.c works).
                           */
+  GIcon *fallback_icon;
 
   ShellAppRunningState *running_state;
 
@@ -180,66 +181,47 @@ window_backed_app_get_window (ShellApp     *app)
     return NULL;
 }
 
-static ClutterActor *
-window_backed_app_get_icon (ShellApp *app,
-                            int       size)
+/**
+ * shell_app_get_icon:
+ *
+ * Look up the icon for this application
+ *
+ * Return value: (transfer none): A #GIcon
+ */
+GIcon *
+shell_app_get_icon (ShellApp *app)
 {
   MetaWindow *window = NULL;
-  StWidget *widget;
-  int scale, scaled_size;
-  ShellGlobal *global;
-  StThemeContext *context;
 
-  global = shell_global_get ();
-  context = st_theme_context_get_for_stage (shell_global_get_stage (global));
-  g_object_get (context, "scale-factor", &scale, NULL);
+  g_return_val_if_fail (SHELL_IS_APP (app), NULL);
 
-  scaled_size = size * scale;
+  if (app->info)
+    return g_app_info_get_icon (G_APP_INFO (app->info));
+
+  if (app->fallback_icon)
+    return app->fallback_icon;
 
   /* During a state transition from running to not-running for
    * window-backend apps, it's possible we get a request for the icon.
-   * Avoid asserting here and just return an empty image.
+   * Avoid asserting here and just return a fallback icon
    */
   if (app->running_state != NULL)
     window = window_backed_app_get_window (app);
 
-  if (window == NULL)
+  if (window &&
+      meta_window_get_client_type (window) == META_WINDOW_CLIENT_TYPE_X11)
     {
-      ClutterActor *actor;
-
-      actor = clutter_actor_new ();
-      g_object_set (actor,
-                    "opacity", 0,
-                    "width", (float) scaled_size,
-                    "height", (float) scaled_size,
-                    NULL);
-      return actor;
-    }
-
-  if (meta_window_get_client_type (window) == META_WINDOW_CLIENT_TYPE_X11)
-    {
-      StWidget *texture_actor;
-
-      texture_actor =
+      app->fallback_icon =
         st_texture_cache_bind_cairo_surface_property (st_texture_cache_get_default (),
                                                       G_OBJECT (window),
-                                                      "icon",
-                                                      scaled_size);
-
-      widget = g_object_new (ST_TYPE_BIN,
-                             "child", texture_actor,
-                             NULL);
+                                                      "icon");
     }
   else
     {
-      widget = g_object_new (ST_TYPE_ICON,
-                             "icon-size", size,
-                             "icon-name", "application-x-executable",
-                             NULL);
+      app->fallback_icon = g_themed_icon_new ("application-x-executable");
     }
-  st_widget_add_style_class_name (widget, "fallback-app-icon");
 
-  return CLUTTER_ACTOR (widget);
+  return app->fallback_icon;
 }
 
 /**
@@ -257,16 +239,16 @@ shell_app_create_icon_texture (ShellApp   *app,
   GIcon *icon;
   ClutterActor *ret;
 
-  if (app->info == NULL)
-    return window_backed_app_get_icon (app, size);
-
   ret = st_icon_new ();
   st_icon_set_icon_size (ST_ICON (ret), size);
   st_icon_set_fallback_icon_name (ST_ICON (ret), "application-x-executable");
 
-  icon = g_app_info_get_icon (G_APP_INFO (app->info));
+  icon = shell_app_get_icon (app);
   st_icon_set_gicon (ST_ICON (ret), icon);
 
+  if (shell_app_is_window_backed (app))
+    st_widget_add_style_class_name (ST_WIDGET (ret), "fallback-app-icon");
+
   return ret;
 }
 
@@ -1576,6 +1558,7 @@ shell_app_dispose (GObject *object)
   ShellApp *app = SHELL_APP (object);
 
   g_clear_object (&app->info);
+  g_clear_object (&app->fallback_icon);
 
   while (app->running_state)
     _shell_app_remove_window (app, app->running_state->windows->data);
