@@ -163,6 +163,23 @@ do_grab_screenshot (ShellScreenshot *screenshot,
   g_free (captures);
 }
 
+static gboolean
+should_draw_cursor_image (ShellScreenshotMode mode)
+{
+  if (mode == SHELL_SCREENSHOT_WINDOW || !meta_is_wayland_compositor ())
+    {
+      StSettings *settings = st_settings_get ();
+      gboolean magnifier_active = FALSE;
+
+      g_object_get (settings, "magnifier-active", &magnifier_active, NULL);
+
+      if (!magnifier_active)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 draw_cursor_image (cairo_surface_t       *surface,
                    cairo_rectangle_int_t  area)
@@ -369,15 +386,6 @@ grab_window_screenshot (ClutterActor *stage,
 
   priv->image = meta_window_actor_get_image (META_WINDOW_ACTOR (window_actor),
                                              NULL);
-
-  if (!priv->image)
-    {
-      g_task_report_new_error (screenshot, on_screenshot_written, result, NULL,
-                               G_IO_ERROR, G_IO_ERROR_FAILED,
-                               "Capturing window failed");
-      return;
-    }
-
   priv->datetime = g_date_time_new_now_local ();
 
   if (priv->include_cursor)
@@ -385,8 +393,7 @@ grab_window_screenshot (ClutterActor *stage,
       if (meta_window_get_client_type (window) == META_WINDOW_CLIENT_TYPE_WAYLAND)
         {
           float resource_scale;
-          if (!clutter_actor_get_resource_scale (window_actor, &resource_scale))
-            resource_scale = 1.0f;
+          resource_scale = clutter_actor_get_resource_scale (window_actor);
 
           cairo_surface_set_device_scale (priv->image, resource_scale, resource_scale);
         }
@@ -460,6 +467,7 @@ shell_screenshot_screenshot (ShellScreenshot     *screenshot,
 {
   ClutterActor *stage;
   ShellScreenshotPrivate *priv;
+  gboolean use_paint_signal = FALSE;
   GTask *result;
 
   g_return_if_fail (SHELL_IS_SCREENSHOT (screenshot));
@@ -484,15 +492,32 @@ shell_screenshot_screenshot (ShellScreenshot     *screenshot,
   g_task_set_source_tag (result, shell_screenshot_screenshot);
 
   priv->stream = g_object_ref (stream);
-  priv->include_cursor = include_cursor;
+  priv->include_cursor = FALSE;
 
   stage = CLUTTER_ACTOR (shell_global_get_stage (priv->global));
 
   meta_disable_unredirect_for_display (shell_global_get_display (priv->global));
 
-  g_signal_connect_after (stage, "actors-painted",
-                          G_CALLBACK (on_actors_painted),
-                          result);
+  if (include_cursor)
+    {
+      if (should_draw_cursor_image (SHELL_SCREENSHOT_SCREEN))
+        priv->include_cursor = TRUE;
+      else
+        use_paint_signal = TRUE;
+    }
+
+  if (use_paint_signal)
+    {
+      g_signal_connect_after (stage, "paint",
+                              G_CALLBACK (on_paint),
+                              result);
+    }
+  else
+    {
+      g_signal_connect_after (stage, "actors-painted",
+                              G_CALLBACK (on_actors_painted),
+                              result);
+    }
 
   clutter_actor_queue_redraw (stage);
 }
@@ -670,7 +695,8 @@ shell_screenshot_screenshot_window (ShellScreenshot     *screenshot,
 
   priv->stream = g_object_ref (stream);
   priv->include_frame = include_frame;
-  priv->include_cursor = include_cursor;
+  priv->include_cursor = include_cursor &&
+                         should_draw_cursor_image (SHELL_SCREENSHOT_WINDOW);
 
   stage = CLUTTER_ACTOR (shell_global_get_stage (priv->global));
 
