@@ -119,6 +119,16 @@ shell_window_tracker_class_init (ShellWindowTrackerClass *klass)
                                                    G_TYPE_NONE, 0);
 }
 
+static gboolean
+check_app_id_prefix (ShellApp   *app,
+                     const char *prefix)
+{
+  if (prefix == NULL)
+    return TRUE;
+
+  return g_str_has_prefix (shell_app_get_id (app), prefix);
+}
+
 /*
  * get_app_from_window_wmclass:
  *
@@ -135,8 +145,10 @@ get_app_from_window_wmclass (MetaWindow  *window)
   ShellAppSystem *appsys;
   const char *wm_class;
   const char *wm_instance;
+  const char *sandbox_id;
 
   appsys = shell_app_system_get_default ();
+  sandbox_id = meta_window_get_sandboxed_app_id (window);
 
   /* Notes on the heuristics used here:
      much of the complexity here comes from the desire to support
@@ -176,23 +188,23 @@ get_app_from_window_wmclass (MetaWindow  *window)
   /* first try a match from WM_CLASS (instance part) to StartupWMClass */
   wm_instance = meta_window_get_wm_class_instance (window);
   app = shell_app_system_lookup_startup_wmclass (appsys, wm_instance);
-  if (app != NULL)
+  if (app != NULL && check_app_id_prefix (app, sandbox_id))
     return g_object_ref (app);
 
   /* then try a match from WM_CLASS to StartupWMClass */
   wm_class = meta_window_get_wm_class (window);
   app = shell_app_system_lookup_startup_wmclass (appsys, wm_class);
-  if (app != NULL)
+  if (app != NULL && check_app_id_prefix (app, sandbox_id))
     return g_object_ref (app);
 
   /* then try a match from WM_CLASS (instance part) to .desktop */
   app = shell_app_system_lookup_desktop_wmclass (appsys, wm_instance);
-  if (app != NULL)
+  if (app != NULL && check_app_id_prefix (app, sandbox_id))
     return g_object_ref (app);
 
   /* finally, try a match from WM_CLASS to .desktop */
   app = shell_app_system_lookup_desktop_wmclass (appsys, wm_class);
-  if (app != NULL)
+  if (app != NULL && check_app_id_prefix (app, sandbox_id))
     return g_object_ref (app);
 
   return NULL;
@@ -214,7 +226,7 @@ get_app_from_id (MetaWindow  *window,
 {
   ShellApp *app;
   ShellAppSystem *appsys;
-  char *desktop_file;
+  g_autofree char *desktop_file = NULL;
 
   g_return_val_if_fail (id != NULL, NULL);
 
@@ -223,10 +235,9 @@ get_app_from_id (MetaWindow  *window,
   desktop_file = g_strconcat (id, ".desktop", NULL);
   app = shell_app_system_lookup_app (appsys, desktop_file);
   if (app)
-    g_object_ref (app);
+    return g_object_ref (app);
 
-  g_free (desktop_file);
-  return app;
+  return NULL;
 }
 
 /*
@@ -391,6 +402,13 @@ get_app_for_window (ShellWindowTracker    *tracker,
   if (meta_window_is_remote (window))
     return _shell_app_new_for_window (window);
 
+  /* Check if the app's WM_CLASS specifies an app; this is
+   * canonical if it does.
+   */
+  result = get_app_from_window_wmclass (window);
+  if (result != NULL)
+    return result;
+
   /* Check if the window was opened from within a sandbox; if this
    * is the case, a corresponding .desktop file is guaranteed to match;
    */
@@ -405,13 +423,6 @@ get_app_for_window (ShellWindowTracker    *tracker,
   if (result != NULL)
     return result;
 
-  /* Check if the app's WM_CLASS specifies an app; this is
-   * canonical if it does.
-   */
-  result = get_app_from_window_wmclass (window);
-  if (result != NULL)
-    return result;
-
   result = get_app_from_window_pid (tracker, window);
   if (result != NULL)
     return result;
