@@ -201,44 +201,42 @@ st_label_paint (ClutterActor        *actor,
 
   if (shadow_spec)
     {
+      ClutterActorBox allocation;
+      float width, height;
       float resource_scale;
 
-      if (clutter_actor_get_resource_scale (priv->label, &resource_scale))
+      clutter_actor_get_allocation_box (priv->label, &allocation);
+      clutter_actor_box_get_size (&allocation, &width, &height);
+
+      resource_scale = clutter_actor_get_resource_scale (priv->label);
+
+      width *= resource_scale;
+      height *= resource_scale;
+
+      if (priv->text_shadow_pipeline == NULL ||
+          width != priv->shadow_width ||
+          height != priv->shadow_height)
         {
-          ClutterActorBox allocation;
-          float width, height;
+          g_clear_pointer (&priv->text_shadow_pipeline, cogl_object_unref);
 
-          clutter_actor_get_allocation_box (priv->label, &allocation);
-          clutter_actor_box_get_size (&allocation, &width, &height);
+          priv->shadow_width = width;
+          priv->shadow_height = height;
+          priv->text_shadow_pipeline =
+            _st_create_shadow_pipeline_from_actor (shadow_spec,
+                                                   priv->label);
+        }
 
-          width *= resource_scale;
-          height *= resource_scale;
+      if (priv->text_shadow_pipeline != NULL)
+        {
+          CoglFramebuffer *framebuffer;
 
-          if (priv->text_shadow_pipeline == NULL ||
-              width != priv->shadow_width ||
-              height != priv->shadow_height)
-            {
-              g_clear_pointer (&priv->text_shadow_pipeline, cogl_object_unref);
-
-              priv->shadow_width = width;
-              priv->shadow_height = height;
-              priv->text_shadow_pipeline =
-                _st_create_shadow_pipeline_from_actor (shadow_spec,
-                                                       priv->label);
-            }
-
-          if (priv->text_shadow_pipeline != NULL)
-            {
-              CoglFramebuffer *framebuffer;
-
-              framebuffer =
-                clutter_paint_context_get_framebuffer (paint_context);
-              _st_paint_shadow_with_opacity (shadow_spec,
-                                             framebuffer,
-                                             priv->text_shadow_pipeline,
-                                             &allocation,
-                                             clutter_actor_get_paint_opacity (priv->label));
-            }
+          framebuffer =
+            clutter_paint_context_get_framebuffer (paint_context);
+          _st_paint_shadow_with_opacity (shadow_spec,
+                                         framebuffer,
+                                         priv->text_shadow_pipeline,
+                                         &allocation,
+                                         clutter_actor_get_paint_opacity (priv->label));
         }
     }
 
@@ -246,11 +244,14 @@ st_label_paint (ClutterActor        *actor,
 }
 
 static void
-st_label_resource_scale_changed (StWidget *widget)
+st_label_resource_scale_changed (ClutterActor *actor)
 {
-  StLabelPrivate *priv = ST_LABEL (widget)->priv;
+  StLabelPrivate *priv = ST_LABEL (actor)->priv;
 
   g_clear_pointer (&priv->text_shadow_pipeline, cogl_object_unref);
+
+  if (CLUTTER_ACTOR_CLASS (st_label_parent_class)->resource_scale_changed)
+    CLUTTER_ACTOR_CLASS (st_label_parent_class)->resource_scale_changed (actor);
 }
 
 static void
@@ -268,9 +269,9 @@ st_label_class_init (StLabelClass *klass)
   actor_class->allocate = st_label_allocate;
   actor_class->get_preferred_width = st_label_get_preferred_width;
   actor_class->get_preferred_height = st_label_get_preferred_height;
+  actor_class->resource_scale_changed = st_label_resource_scale_changed;
 
   widget_class->style_changed = st_label_style_changed;
-  widget_class->resource_scale_changed = st_label_resource_scale_changed;
   widget_class->get_accessible_type = st_label_accessible_get_type;
 
   props[PROP_CLUTTER_TEXT] =
@@ -293,6 +294,7 @@ st_label_class_init (StLabelClass *klass)
 static void
 st_label_init (StLabel *label)
 {
+  ClutterActor *actor = CLUTTER_ACTOR (label);
   StLabelPrivate *priv;
 
   label->priv = priv = st_label_get_instance_private (label);
@@ -304,7 +306,10 @@ st_label_init (StLabel *label)
   label->priv->shadow_width = -1.;
   label->priv->shadow_height = -1.;
 
-  clutter_actor_add_child (CLUTTER_ACTOR (label), priv->label);
+  clutter_actor_add_child (actor, priv->label);
+
+  clutter_actor_set_offscreen_redirect (actor,
+                                        CLUTTER_OFFSCREEN_REDIRECT_ALWAYS);
 }
 
 /**
