@@ -124,7 +124,6 @@ enum
 {
   STYLE_CHANGED,
   POPUP_MENU,
-  RESOURCE_SCALE_CHANGED,
 
   LAST_SIGNAL
 };
@@ -412,8 +411,7 @@ st_widget_paint_background (StWidget            *widget,
   float resource_scale;
   guint8 opacity;
 
-  if (!st_widget_get_resource_scale (widget, &resource_scale))
-    return;
+  resource_scale = clutter_actor_get_resource_scale (CLUTTER_ACTOR (widget));
 
   framebuffer = clutter_paint_context_get_framebuffer (paint_context);
   theme_node = st_widget_get_theme_node (widget);
@@ -601,8 +599,11 @@ st_widget_get_theme_node (StWidget *widget)
 
       if (stage == NULL)
         {
+          g_autofree char *desc = st_describe_actor (CLUTTER_ACTOR (widget));
+
           g_critical ("st_widget_get_theme_node called on the widget %s which is not in the stage.",
-                      st_describe_actor (CLUTTER_ACTOR (widget)));
+                      desc);
+
           return g_object_new (ST_TYPE_THEME_NODE, NULL);
         }
 
@@ -817,6 +818,20 @@ st_widget_real_get_focus_chain (StWidget *widget)
   return g_list_reverse (visible);
 }
 
+static void
+st_widget_resource_scale_changed (ClutterActor *actor)
+{
+  StWidget *widget = ST_WIDGET (actor);
+  StWidgetPrivate *priv = st_widget_get_instance_private (widget);
+  int i;
+
+  for (i = 0; i < G_N_ELEMENTS (priv->paint_states); i++)
+    st_theme_node_paint_state_invalidate (&priv->paint_states[i]);
+
+  if (CLUTTER_ACTOR_CLASS (st_widget_parent_class)->resource_scale_changed)
+    CLUTTER_ACTOR_CLASS (st_widget_parent_class)->resource_scale_changed (actor);
+}
+
 static void
 st_widget_class_init (StWidgetClass *klass)
 {
@@ -846,6 +861,8 @@ st_widget_class_init (StWidgetClass *klass)
   actor_class->get_accessible = st_widget_get_accessible;
   actor_class->has_accessible = st_widget_has_accessible;
 
+  actor_class->resource_scale_changed = st_widget_resource_scale_changed;
+
   klass->style_changed = st_widget_real_style_changed;
   klass->navigate_focus = st_widget_real_navigate_focus;
   klass->get_accessible_type = st_widget_accessible_get_type;
@@ -999,21 +1016,6 @@ st_widget_class_init (StWidgetClass *klass)
                   G_STRUCT_OFFSET (StWidgetClass, popup_menu),
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
-
-  /**
-   * StWidget::resource-scale-changed:
-   * @widget: the #StWidget
-   *
-   * Emitted when the paint scale that the widget will be painted as
-   * changed.
-   */
-  signals[RESOURCE_SCALE_CHANGED] =
-    g_signal_new ("resource-scale-changed",
-                  G_TYPE_FROM_CLASS (klass),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (StWidgetClass, resource_scale_changed),
-                  NULL, NULL, NULL,
-                  G_TYPE_NONE, 0);
 }
 
 static const gchar *
@@ -1397,23 +1399,6 @@ st_widget_get_style (StWidget *actor)
   return ST_WIDGET_PRIVATE (actor)->inline_style;
 }
 
-/**
- * st_widget_get_resource_scale:
- * @widget: A #StWidget
- * @resource_scale: (out): return location for the resource scale
- *
- * Retrieves the resource scale for this #StWidget, if available.
- *
- * The resource scale refers to the scale the actor should use for its resources.
- */
-gboolean
-st_widget_get_resource_scale (StWidget *widget,
-                              float    *resource_scale)
-{
-  return clutter_actor_get_resource_scale (CLUTTER_ACTOR (widget),
-                                           resource_scale);
-}
-
 static void
 st_widget_set_first_visible_child (StWidget     *widget,
                                    ClutterActor *actor)
@@ -1480,23 +1465,6 @@ st_widget_name_notify (StWidget   *widget,
   st_widget_style_changed (widget);
 }
 
-static void
-st_widget_resource_scale_notify (StWidget   *widget,
-                                 GParamSpec *pspec,
-                                 gpointer    data)
-{
-  StWidgetPrivate *priv = st_widget_get_instance_private (widget);
-  int i;
-
-  for (i = 0; i < G_N_ELEMENTS (priv->paint_states); i++)
-    st_theme_node_paint_state_invalidate (&priv->paint_states[i]);
-
-  g_signal_emit (widget, signals[RESOURCE_SCALE_CHANGED], 0);
-
-  if (clutter_actor_is_mapped (CLUTTER_ACTOR (widget)))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR (widget));
-}
-
 static void
 st_widget_reactive_notify (StWidget   *widget,
                            GParamSpec *pspec,
@@ -1648,7 +1616,6 @@ st_widget_init (StWidget *actor)
 
   /* connect style changed */
   g_signal_connect (actor, "notify::name", G_CALLBACK (st_widget_name_notify), NULL);
-  g_signal_connect (actor, "notify::resource-scale", G_CALLBACK (st_widget_resource_scale_notify), NULL);
   g_signal_connect (actor, "notify::reactive", G_CALLBACK (st_widget_reactive_notify), NULL);
 
   g_signal_connect (actor, "notify::visible", G_CALLBACK (st_widget_visible_notify), NULL);
@@ -1720,7 +1687,8 @@ st_widget_recompute_style (StWidget    *widget,
            */
 
           priv->transition_animation =
-            st_theme_node_transition_new (old_theme_node,
+            st_theme_node_transition_new (CLUTTER_ACTOR (widget),
+                                          old_theme_node,
                                           new_theme_node,
                                           current_paint_state (widget),
                                           transition_duration);
@@ -1903,7 +1871,7 @@ st_widget_sync_hover (StWidget *widget)
 
   seat = clutter_backend_get_default_seat (clutter_get_default_backend ());
   pointer = clutter_seat_get_pointer (seat);
-  pointer_actor = clutter_input_device_get_pointer_actor (pointer);
+  pointer_actor = clutter_input_device_get_actor (pointer, NULL);
   if (pointer_actor && clutter_actor_get_reactive (CLUTTER_ACTOR (widget)))
     st_widget_set_hover (widget, clutter_actor_contains (CLUTTER_ACTOR (widget), pointer_actor));
   else
