@@ -63,7 +63,6 @@ struct _StWidgetClass
   /* signals */
   void     (* style_changed)       (StWidget         *self);
   void     (* popup_menu)          (StWidget         *self);
-  void     (* resource_scale_changed) (StWidget         *self);
 
   /* vfuncs */
 
@@ -135,8 +134,6 @@ StThemeNode *         st_widget_peek_theme_node           (StWidget        *widg
 GList *               st_widget_get_focus_chain           (StWidget        *widget);
 void                  st_widget_paint_background          (StWidget            *widget,
                                                            ClutterPaintContext *paint_context);
-gboolean              st_widget_get_resource_scale        (StWidget        *widget,
-                                                           float           *resource_scale);
 
 /* debug methods */
 char  *st_describe_actor       (ClutterActor *actor);
